// 导出
import tableExport from '../table-export.vue'
// 分页
import tablePage from '../table-page.vue'
// 头部
import tableTop from '../table-top.vue'
import '../assets/table.scss'
export default {
    props: {
        layoutInfo: {
            type: [Array]
        },

        isDrag: {
            type: Boolean,
            default: false
        },

        pageChangeFn: {
            type: Function
        },

        // 是否支持多级表头
        isMultiHeader: {
            type: Boolean,
            default: false
        },

        // 选项扩充
        optionList: {
            type: Array,
            default: () => []
        },

        appendNum: {
            type: Number,
            default: 80
        },

        isShowHeader: {
            type: Boolean,
            default: false
        },

        isShowSelect: {
            type: Boolean,
            default: false
        },

        isShowPage: {
            type: Boolean,
            default: false
        },

        freezeColumns: {
            type: Array
        },

        pageInfo: {
            type: Object
        },

        summaryData: {
            type: Array
        },

        isSortMultiple: {
            type: [Boolean],
            default: false
        },
        isShowSortMultiple: {
            type: [Boolean],
            default: false
        },
        noHeight: {
            type: [Boolean],
            default: false
        },
        isShowTotal: {
            type: Boolean,
            default: false
        }
    },
    data() {
        return {
            dialogVisible: false,
            isShowHead: false
        }
    },
    created() {
        if (this.freezeColumns) {
            this.store.states.data = this.handleData(this.store.states.data, this.freezeColumns)
        }
        this.isShowHead = this.isShowHeader
        if (this.isShowSelect) {
            this.isShowHead = true
        }
    },
    watch: {
        showHeader(val, oVal) {
            if (val !== oVal) {
                this.doLayout()
            }
        }
    },
    components: {
        tableExport,
        tablePage,
        tableTop
    },
    methods: {
        reset() {
            const that = this
            that.$confirm(that.t('el.table.option.resettxt'), that.t('el.table.option.confirm'), {
                distinguishCancelAndClose: true,
                confirmButtonText: that.t('el.table.option.yes'),
                cancelButtonText: that.t('el.table.option.no'),
            }).then(() => {
                that.$message({
                    type: 'success',
                    message: that.t('el.table.option.tishi')
                });
                //重置列属性
                const fn = it => {
                    it.index = it.oIndex
                    it.width = it.oWidth
                    // 需要处理
                    it.isHideColumn ? (it.isSelected = false) : (it.isSelected = true)
                }
                that.store.states.resetColumns.forEach(fn)

                if (that.layoutInfo) {
                    that.layoutInfo.length = 0
                }
                that.$emit('layout-reset', that.store.states.fullColumns.map(that.submitFn))
                if (!that.isMultiHeader) {
                    that.store.states.originColumns = that.store.states.columns = that.handleArr(that.store.states.resetColumns)
                } else {
                    that.store.states.columns = that.handleArr(that.store.states.resetColumns)
                }
                setTimeout(() => {
                    that.doLayout()
                }, 50)
            })
        },
        closeAndClear() {
            this.$refs['table-export'].dialogVisible = false;
        },
        getCotrolColumns(arr) {
            let that = this
            let controlArr = []
            that.store.states.columns.forEach(item => {
                if (this.isExist(item, arr)) {
                    controlArr.push(item)
                }
            })
            return controlArr
        },
        isExist(item, arr) {
            for (let i = 0; i < arr.length; i++) {
                if (arr[i].property == item.property && arr[i].label == item.label) {
                    item.width = arr[i].width
                    return true
                }
            }
            return false
        },
        // 数据冻结
        handleData(data, columns) {
            let arr = JSON.parse(JSON.stringify(data))
            if (columns && columns.length) {
                columns.forEach(key => {
                    arr.forEach(item => {
                        if (key in item) {
                            let value = item[key]
                            delete item[key]
                            Object.defineProperty(item, key, {
                                value: value,
                                enumerable: false,
                                writable: false,
                                configurable: false
                            })
                        }
                    })
                })
            } else {
                Object.freeze(arr) // freeze  preventExtensions seal
            }
            return arr
        },
        sizeChange(event) {
            this.$emit('page-size-change', event);
        },
        currentChange(event) {
            this.$emit('page-current-change', event);
        },
        doExport(event) {
            this.$emit('do-export', event);
        },
        close() {
            this.$refs['table-export'].dialogVisible = false;
        },
        getClass() {
            let str = ''
            if (this.isShowHead) {
                str += 'zw';
            }
            if (this.isShowPage) {
                str += ' zw-page';
            }
            return str;
        },
        openDialog() {
            this.$refs['table-export'].dialogVisible = true;
        }
    }
}