import { getCell, getColumnByCell, getRowIdentity } from './table/util';
import { hasClass, addClass, removeClass } from '../../../src/utils/dom';
import ElCheckbox from '../../checkbox';
import debounce from 'throttle-debounce/debounce';
import LayoutObserver from './table/layout-observer';
import tableBodyMixin from './table/table-body'
export default {
    name: 'ElTableBody',

    mixins: [LayoutObserver, tableBodyMixin],

    components: {
        ElCheckbox
    },

    props: {
        tableData: Array,
        times0: Number,
        times1: Number,
        times2: Number,
        groupIndex: Number,
        itemNum: Number,
        tableIndex: Number,
        itemRowHeight: {
            type: Number,
            default: 28
        }
    },

    render(h) {
        const columnsHidden = this.columns.map((column, index) => this.isColumnHidden(index));
        return (
            <table
                ref="tableBody"
                class="el-table__body"
                cellspacing="0"
                cellpadding="0"
                border="0">
                <colgroup>
                    {
                        this._l(this.columns, column => <col name={column.id} />)
                    }
                </colgroup>
                <tbody>
                    {
                        this._l(this.timesTableData, (row, $index) =>
                            [<tr
                                style={this.rowStyle ? this.getRowStyle(row, this.fvIndex($index, row)) : null}
                                key={this.table.rowKey ? this.getKeyOfRow(row, this.fvIndex($index, row)) : this.fvIndex($index, row)}
                                on-dblclick={($event) => this.handleDoubleClick($event, row)}
                                on-click={($event) => this.handleClick($event, row)}
                                on-contextmenu={($event) => this.handleContextMenu($event, row)}
                                on-mouseenter={() => this.handleMouseEnter(this.fvIndex($index, row))}
                                on-mouseleave={() => this.handleMouseLeave()}
                                class={[this.getRowClass(row, this.fvIndex($index, row))]}>
                                {
                                    this._l(this.columns, (column, cellIndex) => {
                                        const { rowspan, colspan } = this.getSpan(row, column, this.fvIndex($index, row), cellIndex);
                                        if (!rowspan || !colspan) {
                                            return '';
                                        } else {
                                            if (rowspan === 1 && colspan === 1) {
                                                return (
                                                    <td
                                                        style={[{ height: this.itemRowHeight + 'px' }, this.getCellStyle(this.fvIndex($index, row), cellIndex, row, column)]}
                                                        class={this.getCellClass(this.fvIndex($index, row), cellIndex, row, column)}
                                                        on-mouseenter={($event) => this.handleCellMouseEnter($event, row)}
                                                        on-mouseleave={this.handleCellMouseLeave}>
                                                        {
                                                            column.renderCell.call(
                                                                this._renderProxy,
                                                                h,
                                                                {
                                                                    row,
                                                                    column,
                                                                    $index: this.fvIndex($index, row),
                                                                    store: this.store,
                                                                    _self: this.context || this.table.$vnode.context
                                                                },
                                                                columnsHidden[cellIndex]
                                                            )
                                                        }
                                                    </td>
                                                );
                                            } else {
                                                return (
                                                    <td
                                                        style={[{ height: this.itemRowHeight + 'px' }, this.getCellStyle(this.fvIndex($index, row), cellIndex, row, column)]}
                                                        class={this.getCellClass(this.fvIndex($index, row), cellIndex, row, column)}
                                                        rowspan={rowspan}
                                                        colspan={colspan}
                                                        on-mouseenter={($event) => this.handleCellMouseEnter($event, row)}
                                                        on-mouseleave={this.handleCellMouseLeave}>
                                                        {
                                                            column.renderCell.call(
                                                                this._renderProxy,
                                                                h,
                                                                {
                                                                    row,
                                                                    column,
                                                                    $index: this.fvIndex($index, row),
                                                                    store: this.store,
                                                                    _self: this.context || this.table.$vnode.context
                                                                },
                                                                columnsHidden[cellIndex]
                                                            )
                                                        }
                                                    </td>
                                                );
                                            }
                                        }
                                    })
                                }
                            </tr>,
                            this.store.isRowExpanded(row)
                                ? (<tr>
                                    <td colspan={this.columns.length} class="el-table__expanded-cell">
                                        {this.table.renderExpanded ? this.table.renderExpanded(h, { row, $index: this.fvIndex($index, row), store: this.store }) : ''}
                                    </td>
                                </tr>)
                                : ''
                            ]
                        ).concat(
                            //   <pl-tooltip effect={ this.table.tooltipEffect } placement="top" ref="tooltip" content={ this.tooltipContent }></pl-tooltip>
                        )
                    }
                </tbody>
            </table>
        );
    },

    computed: {
        //  取数据
        timesTableData() {
            let data = [];
            let count1 = 0;
            let count2 = 0;
            let count3 = 0;
            this.data.filter((e, i) => {
                e.initRowIndex = i;
            });
            switch (this.tableIndex) {
                case 1:
                    count1 = this.times0 * this.itemNum * 3;
                    data = this.data.slice(count1, count1 + this.itemNum);
                    break;
                case 2:
                    count2 = this.times1 * this.itemNum * 3;
                    data = this.data.slice(count2 + this.itemNum, count2 + this.itemNum * 2);
                    break;
                case 3:
                    count3 = this.times2 * this.itemNum * 3;
                    data = this.data.slice(count3 + this.itemNum * 2, count3 + this.itemNum * 3);
                    break;
            }
            //  选中行
            this.$nextTick(() => {
                this.dfCurrentRow(this.store.states.currentRow);
            });

            return data;
        }
    },

    data() {
        return {
            //  定时器监听table高度改变
            intervalId: null,
            //  当前table高度
            tableBodyHeight: 0
        };
    },

    mounted() {
        this.$nextTick(() => {
            if (this.$refs.tableBody && this.$refs.tableBody.offsetHeight !== undefined) {
                this.tableBodyHeight = this.$refs.tableBody.offsetHeight;
                //  定时器监听当前表格高度改变
                this.intervalId = setInterval(() => {
                    if (this.$refs.tableBody.offsetHeight > 0 && this.tableBodyHeight !== this.$refs.tableBody.offsetHeight) {
                        //  有改变
                        this.tableBodyHeight = this.$refs.tableBody.offsetHeight;
                        //  当前块取当前分组
                        let groupIndex = this.times0;
                        switch (this.tableIndex) {
                            case 1:
                                groupIndex = this.times0 * 3 + this.tableIndex - 1;
                                break;
                            case 2:
                                groupIndex = this.times1 * 3 + this.tableIndex - 1;
                                break;
                            case 3:
                                groupIndex = this.times2 * 3 + this.tableIndex - 1;
                                break;
                        }
                        // 执行修改高度与总高度
                        this.$emit('changeHeight', groupIndex, this.tableBodyHeight);
                    }
                }, 80);
            }
        }
        );
    },
    destroyed() {
        if (this.intervalId) {
            clearInterval(this.intervalId);
        }
    },
    methods: {
        //  $index 处理
        fvIndex(index, item) {
            let _index = index;
            if (item && 'initRowIndex' in item) {
                _index = item['initRowIndex'];
            }
            return _index;
        },
        //  默认选中行处理
        dfCurrentRow(currentRow) {
            if (!this.highlight) return;
            const el = this.$el;
            if (!el) return;
            // const data = this.store.states.data;
            const data = this.timesTableData;
            const tr = el.querySelector('tbody').children;
            const rows = [].filter.call(tr, row => hasClass(row, 'el-table__row'));
            const oldRow = rows[data.indexOf(currentRow)];
            const newRow = rows[data.indexOf(currentRow)];
            if (oldRow) {
                removeClass(oldRow, 'current-row');
            } else {
                [].forEach.call(rows, row => removeClass(row, 'current-row'));
            }
            if (newRow) {
                addClass(newRow, 'current-row');
            }
        },
        handleCellMouseEnter(event, row) {
            const table = this.table;
            const cell = getCell(event);

            if (cell) {
                const column = getColumnByCell(table, cell);
                const hoverState = table.hoverState = { cell, column, row };
                table.$emit('cell-mouse-enter', hoverState.row, hoverState.column, hoverState.cell, event);
            }

            // 判断是否text-overflow, 如果是就显示tooltip
            const cellChild = event.target.querySelector('.cell');
            if (hasClass(cellChild, 'el-tooltip') && cellChild.scrollWidth > cellChild.offsetWidth) {
                // TODO 会引起整个 Table 的重新渲染，需要优化
                this.tooltipContent = cell.textContent || cell.innerText;
                cellChild && (cellChild.setAttribute("title", this.tooltipContent));
            }
        }
    }
};
