import tableFooter from './table/table-footer'
export default {
    render(h) {
        let sums = [];
        if (this.summaryMethod) {
            sums = this.summaryMethod({ columns: this.columns, data: this.store.states.data });
        } else {
            this.columns.forEach((column, index) => {
                if (index === 0) {
                    sums[index] = this.sumText;
                    return;
                }
                const values = this.store.states.data.map(item => Number(item[column.property]));
                const precisions = [];
                let notNumber = true;
                values.forEach(value => {
                    if (!isNaN(value)) {
                        notNumber = false;
                        let decimal = ('' + value).split('.')[1];
                        precisions.push(decimal ? decimal.length : 0);
                    }
                });
                const precision = Math.max.apply(null, precisions);
                if (!notNumber) {
                    sums[index] = values.reduce((prev, curr) => {
                        const value = Number(curr);
                        if (!isNaN(value)) {
                            return parseFloat((prev + curr).toFixed(Math.min(precision, 20)));
                        } else {
                            return prev;
                        }
                    }, 0);
                } else {
                    sums[index] = '';
                }
            });
        }

        return (
            <div>
                <table
                    class="el-table__footer"
                    cellspacing="0"
                    cellpadding="0"
                    border="0">
                    <colgroup>
                        {
                            this._l(this.columns, column => <col name={column.id} />)
                        }
                        {
                            this.hasGutter ? <col name="gutter" /> : ''
                        }
                    </colgroup>
                    <tbody class={[{ 'has-gutter': this.hasGutter }]}>
                        <tr>
                            {
                                this._l(this.columns, (column, cellIndex) =>
                                    <td
                                        colspan={column.colSpan}
                                        rowspan={column.rowSpan}
                                        class={[column.id, column.headerAlign, column.className || '', this.isCellHidden(cellIndex, this.columns) ? 'is-hidden' : '', !column.children ? 'is-leaf' : '', column.labelClassName]}>
                                        <div class={['cell', column.labelClassName]}>
                                            {
                                                sums[cellIndex]
                                            }
                                        </div>
                                    </td>
                                )
                            }
                            {
                                this.hasGutter ? <th class="gutter"></th> : ''
                            }
                        </tr>
                    </tbody>
                </table>
                {
                    this.isShowTotal ? (
                        <table
                            class="el-table__footer"
                            cellspacing="0"
                            cellpadding="0"
                            border="0">
                            <colgroup>
                                {
                                    this._l(this.columns, column => <col name={column.id} />)
                                }
                                {
                                    this.hasGutter ? <col name="gutter" /> : ''
                                }
                            </colgroup>
                            <tbody class={[{ 'has-gutter': this.hasGutter }]}>
                                <tr>
                                    {
                                        this._l(this.columns, (column, cellIndex) =>
                                            <td
                                                colspan={column.colSpan}
                                                rowspan={column.rowSpan}
                                                class={[column.id, column.headerAlign, column.className || '', this.isCellHidden(cellIndex, this.columns) ? 'is-hidden' : '', !column.children ? 'is-leaf' : '', column.labelClassName]}>
                                                <div class={['cell', column.labelClassName]}>
                                                    {
                                                        this.totals[cellIndex]
                                                    }
                                                </div>
                                            </td>
                                        )
                                    }
                                    {
                                        this.hasGutter ? <th class="gutter"></th> : ''
                                    }
                                </tr>
                            </tbody>
                        </table>
                    ) : ''
                }
            </div>
        );
    },
    mixins: [tableFooter],
    props: {
        summaryData: {
            type: Array
        },
        totalText: String,
        isShowTotal: {
            type: Boolean,
            default: false
        }
    },
    computed: {
        totals() {
            const total = [];
            const that = this
            that.columns.forEach((column, index) => {
                if (index === 0) {
                    total[index] = this.totalText;
                    return;
                }
                total[index] = that.getSummaryData(column)
            });
            return total;
        }
    },
    methods: {
        getSummaryData(column) {
            if (this.summaryData && this.summaryData.length) {
                for (let i = 0; i < this.summaryData.length; i++) {
                    if (column.property == this.summaryData[i].key) {
                        return this.summaryData[i].value
                    }
                }
            }
            return ''
        }
    }
}