/**
 * desc.:     业务组件 - （弹出框，编辑框）
 *author:     wu.df
 *date:       2018-12-28
 */
import {getLocal} from '../i18n'
export default {
    name: "PlDialogEdit",

    inheritAttrs: false,

    props: {
        title: String,

        closeOnClickModal: {
            type: Boolean,
            default: false,
        },

        visible: {
            type: Boolean,
            default: false,
        },
        top: {
            type: String,
            default: '15px'
        },
        saveData: Function,
        showKeepAdd: {
            type: Boolean,
            default: true
        },
        hasFooter: {
            type: Boolean,
            default: true
        },
        openModel: {
            type: String,
            default: 'add',
        }
    },

    data() {
        return {
            isVisible: false,
            elTop: '',
            isAdd: true,
            keepAdd: false,
            doing: false,
            saveText: this.t('add'),
            titleText: ''
        }
    },

    watch: {
        visible: {
            immediate: true,
            handler: function(val) {
                if (val) {
                    this.show();
                } else {
                    this.isVisible = false;
                }
            }
        }
    },

    computed: {
        pageController() {
            let parent = this.$parent;
            while (parent.$parent && !parent.hasOwnProperty('controller')) {
                parent = parent.$parent;
            }
            return parent;
        }
    },

    methods: {

        t(key) {
            return getLocal(key) || key;    
        },

        show() {
            var me = this;
            me.isVisible = true;

            if (!me.pageController) return;

            setTimeout(() => {
                const obj = me.pageController.controller === 'simple' ? me.pageController.editObject : me.pageController.dtlObject,
                    key = me.pageController.controller === 'simple' ? me.pageController.keyField : me.pageController.dtlKeyField;

                me.isAdd = obj[key] ? false : true;
                me.saveText = me.isAdd ? me.t('add') : me.t('save');
                me.titleText = me.title ? me.title : (me.isAdd ? me.t('add') : me.openModel === 'view' ? me.t('detail') : me.t('edit'));
                const _top = window.innerHeight - $(me.$el).find('.pl-dialog-wrapper').height();
                me.elTop = _top < parseInt(me.top) ? '0px' : me.top;
            }, 10);
        },
        beforeClose() {
            this.close();
        },

        close() {
            this.isVisible = false;
            this.$emit('update:visible', false);
        },

        //取 form 不能放在computed中，因为computed是缓存的
        getForm() {

            if (!this.$refs && !this.$refs.dialog) return null;
            const children = this.$refs.dialog.$children,
                names = ['plform', 'plformpanel', 'elform'];
            for (const child of children) {
                if (names.indexOf(child.$options.name.toLowerCase()) > -1) {
                    return child;
                }
            }
            return null;
        },

        onSave() {
            const me = this,
                form = me.getForm();

            if (!me.pageController || me.doing) return;

            if (!form) { console.log(me.t('userForm')); return; }

            if (!me.saveData) { console.log(me.t('notSaveMethod')); return; }

            form.validate((isPass) => {
                if (!isPass) return;
                // me.doing = true;
                me.saveData(result => {
                    me.doing = false;
                    if (!result) return;
                    const code = (result.flag || {}).retCode;
                    if (me.isAdd && me.keepAdd && code === '0') { //当新增且连续新增且保存成功后，重置form
                        form.resetFields();
                    } else if (code === '0') { //保存成功才关闭窗口。
                        me.close();
                    }
                });
            })
        }
    },

    destroyed() {
        if (this.$refs && this.$refs.dialog) {
            this.$refs.dialog.destroyed();
        }
    }
}